@@ -205,7 +205,7 @@ static void rtree_traverse_recursive(rtree_t * node,
     return;
   }
 
-  rand_double = erand48(rstate);
+  rand_double = mptp_erand48(rstate);
   if (rand_double >= 0.5)
   {
     rtree_traverse_recursive(node->left, cbtrav, index, rstate, outbuffer);
@@ -450,7 +450,7 @@ static rtree_t ** rtree_tipstring_nodes(rtree_t * root,
                                                  sizeof(rtree_t *));
 
   /* create a hashtable of tip labels */
-  hashtable_t * ht = hashtable_create(root->leaves);
+  hashtable_t * ht = hashtable_create((unsigned long)(root->leaves));
 
   for (i = 0; i < (unsigned int)(root->leaves); ++i)
   {
